@@ -21,11 +21,15 @@
 #ifndef EXTRACTOR_H
 #define EXTRACTOR_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * 0.2.6-1 => 0x00020601
  * 4.5.2-0 => 0x04050200
  */
-#define EXTRACTOR_VERSION 0x00030300
+#define EXTRACTOR_VERSION 0x00031100
 
 #include <stdio.h>
 
@@ -36,8 +40,6 @@
    preserved */
 #define EXTRACTOR_DUPLICATES_REMOVE_UNKNOWN 2
 
-
-
 /**
  * Enumeration defining various sources of keywords.
  * See also
@@ -96,6 +98,23 @@ typedef enum {
   EXTRACTOR_BUILDHOST = 49,
   EXTRACTOR_OS = 50,
   EXTRACTOR_DEPENDENCY = 51,
+  EXTRACTOR_HASH_MD4 = 52,
+  EXTRACTOR_HASH_MD5 = 53,
+  EXTRACTOR_HASH_SHA0 = 54,
+  EXTRACTOR_HASH_SHA1 = 55,
+  EXTRACTOR_HASH_RMD160 = 56,
+  EXTRACTOR_RESOLUTION = 57,
+  EXTRACTOR_CATEGORY = 58,
+  EXTRACTOR_BOOKTITLE = 59,
+  EXTRACTOR_PRIORITY = 60,
+  EXTRACTOR_CONFLICTS = 61,
+  EXTRACTOR_REPLACES = 62,
+  EXTRACTOR_PROVIDES = 63,
+  EXTRACTOR_CONDUCTOR = 64,
+  EXTRACTOR_INTERPRET = 65,
+  EXTRACTOR_OWNER = 66,
+  EXTRACTOR_LYRICS = 67,
+  EXTRACTOR_MEDIA_TYPE = 68,
 } EXTRACTOR_KeywordType;
 
 /**
@@ -171,8 +190,9 @@ EXTRACTOR_KeywordType EXTRACTOR_getHighestKeywordTypeNumber();
  * @return the new list of libraries, equal to prev iff an error occured
  *         or if config was empty (or NULL).
  */
-EXTRACTOR_ExtractorList * EXTRACTOR_loadConfigLibraries(EXTRACTOR_ExtractorList * prev,
-							const char * config);
+EXTRACTOR_ExtractorList * 
+EXTRACTOR_loadConfigLibraries(EXTRACTOR_ExtractorList * prev,
+			      const char * config);
 
 /**
  * Add a library for keyword extraction. 
@@ -180,8 +200,9 @@ EXTRACTOR_ExtractorList * EXTRACTOR_loadConfigLibraries(EXTRACTOR_ExtractorList
  * @param library the name of the library
  * @return the new list of libraries, equal to prev iff an error occured
  */
-EXTRACTOR_ExtractorList * EXTRACTOR_addLibrary(EXTRACTOR_ExtractorList * prev,
-					       const char * library);
+EXTRACTOR_ExtractorList * 
+EXTRACTOR_addLibrary(EXTRACTOR_ExtractorList * prev,
+		     const char * library);
 
 /**
  * Add a library for keyword extraction at the END of the list.
@@ -190,8 +211,9 @@ EXTRACTOR_ExtractorList * EXTRACTOR_addLibrary(EXTRACTOR_ExtractorList * prev,
  * @return the new list of libraries, always equal to prev
  *         except if prev was NULL and no error occurs
  */
-EXTRACTOR_ExtractorList * EXTRACTOR_addLibraryLast(EXTRACTOR_ExtractorList * prev,
-						   const char * library);
+EXTRACTOR_ExtractorList * 
+EXTRACTOR_addLibraryLast(EXTRACTOR_ExtractorList * prev,
+			 const char * library);
 
 /**
  * Remove a library for keyword extraction.
@@ -199,8 +221,9 @@ EXTRACTOR_ExtractorList * EXTRACTOR_addLibraryLast(EXTRACTOR_ExtractorList * pre
  * @param library the name of the library to remove
  * @return the reduced list, unchanged if the library was not loaded
  */
-EXTRACTOR_ExtractorList * EXTRACTOR_removeLibrary(EXTRACTOR_ExtractorList * prev,
-						  const char * library);
+EXTRACTOR_ExtractorList * 
+EXTRACTOR_removeLibrary(EXTRACTOR_ExtractorList * prev,
+			const char * library);
 
 /**
  * Remove all extractors.
@@ -215,8 +238,9 @@ void EXTRACTOR_removeAll(EXTRACTOR_ExtractorList * libraries);
  * @return the list of keywords found in the file, NULL if none
  *         were found (or other errors)
  */
-EXTRACTOR_KeywordList * EXTRACTOR_getKeywords(EXTRACTOR_ExtractorList * extractor,
-					      const char * filename);
+EXTRACTOR_KeywordList * 
+EXTRACTOR_getKeywords(EXTRACTOR_ExtractorList * extractor,
+		      const char * filename);
 
 
 /**
@@ -225,8 +249,9 @@ EXTRACTOR_KeywordList * EXTRACTOR_getKeywords(EXTRACTOR_ExtractorList * extracto
  * @param options a set of options (DUPLICATES_XXXX)
  * @return a list of keywords without duplicates
  */ 
-EXTRACTOR_KeywordList * EXTRACTOR_removeDuplicateKeywords(EXTRACTOR_KeywordList * list,
-							  const unsigned int options);
+EXTRACTOR_KeywordList * 
+EXTRACTOR_removeDuplicateKeywords(EXTRACTOR_KeywordList * list,
+				  const unsigned int options);
 
 
 /**
@@ -285,4 +310,8 @@ const char * EXTRACTOR_extractLastByString (const char * type,
 unsigned int EXTRACTOR_countKeywords(EXTRACTOR_KeywordList * keywords);
 
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
