@@ -18,26 +18,15 @@
      Boston, MA 02111-1307, USA.
  */
 
-#include "extractor_util.h"
-#ifdef USE_NS_MODULE
-#include <mach-o/dyld.h>
-#else
+#include "platform.h"
+#include "extractor.h"
+
 #if HAVE_LTDL_H
 #include <ltdl.h>
 #else
 #include <../../libltdl/ltdl.h>
 #endif
-#endif
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-#include <unistd.h>
-#ifndef MINGW
-  #include <sys/mman.h>
-#endif
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
+#include <locale.h>
 
 #define DEBUG 1
 
@@ -45,107 +34,140 @@
  * The sources of keywords as strings.
  */
 static const char *keywordTypes[] = {
-  "unknown",
-  "filename",
-  "mimetype",
-  "title",
-  "author",
-  "artist",
-  "description",
-  "comment",
-  "date",
-  "publisher",
-  "language",
-  "album",
-  "genre",
-  "location",
-  "version",
-  "organization",
-  "copyright",
-  "subject",
-  "keywords",
-  "contributor",
-  "resource-type",
-  "format",
-  "resource-identifier",
-  "source",
-  "relation",
-  "coverage",
-  "software",
-  "disclaimer",
-  "warning",
-  "translated",
-  "creation date",
-  "modification date",
-  "creator",
-  "producer",
-  "page count",
-  "page orientation",
-  "paper size",
-  "used fonts",
-  "page order",
-  "created for",
-  "magnification",
-  "release",
-  "group",
-  "size", 
-  "summary",
-  "packager",
-  "vendor",
-  "license",
-  "distribution",
-  "build-host",
-  "os",
-  "dependency",
+  gettext_noop("unknown"),
+  gettext_noop("filename"),
+  gettext_noop("mimetype"),
+  gettext_noop("title"),
+  gettext_noop("author"),
+  gettext_noop("artist"),
+  gettext_noop("description"),
+  gettext_noop("comment"),
+  gettext_noop("date"),
+  gettext_noop("publisher"),
+  gettext_noop("language"),
+  gettext_noop("album"),
+  gettext_noop("genre"),
+  gettext_noop("location"),
+  gettext_noop("version"),
+  gettext_noop("organization"),
+  gettext_noop("copyright"),
+  gettext_noop("subject"),
+  gettext_noop("keywords"),
+  gettext_noop("contributor"),
+  gettext_noop("resource-type"),
+  gettext_noop("format"),
+  gettext_noop("resource-identifier"),
+  gettext_noop("source"),
+  gettext_noop("relation"),
+  gettext_noop("coverage"),
+  gettext_noop("software"),
+  gettext_noop("disclaimer"),
+  gettext_noop("warning"),
+  gettext_noop("translated"),
+  gettext_noop("creation date"),
+  gettext_noop("modification date"),
+  gettext_noop("creator"),
+  gettext_noop("producer"),
+  gettext_noop("page count"),
+  gettext_noop("page orientation"),
+  gettext_noop("paper size"),
+  gettext_noop("used fonts"),
+  gettext_noop("page order"),
+  gettext_noop("created for"),
+  gettext_noop("magnification"),
+  gettext_noop("release"),
+  gettext_noop("group"),
+  gettext_noop("size"), 
+  gettext_noop("summary"),
+  gettext_noop("packager"),
+  gettext_noop("vendor"),
+  gettext_noop("license"),
+  gettext_noop("distribution"),
+  gettext_noop("build-host"),
+  gettext_noop("os"),
+  gettext_noop("dependency"),
+  gettext_noop("MD4"),
+  gettext_noop("MD5"),
+  gettext_noop("SHA-0"),
+  gettext_noop("SHA-1"),
+  gettext_noop("RipeMD160"),
+  gettext_noop("resolution"),
+  gettext_noop("category"),
+  gettext_noop("book title"),
+  gettext_noop("priority"),
+  gettext_noop("conflicts"),
+  gettext_noop("replaces"),
+  gettext_noop("provides"),
+  gettext_noop("conductor"),
+  gettext_noop("interpreter"),
+  gettext_noop("owner"),
+  gettext_noop("lyrics"),
+  gettext_noop("media type"),
   NULL,
 };
 
 /* the number of keyword types (for bounds-checking) */
-#define HIGHEST_TYPE_NUMBER 52
+#define HIGHEST_TYPE_NUMBER 69
 
 #ifdef HAVE_LIBOGG
-#ifdef HAVE_VORBIS
+#if HAVE_VORBIS
 #define WITH_OGG 1
 #endif
 #endif
 
-#ifdef HAVE_VORBISFILE
+#if HAVE_VORBISFILE
 #define WITH_OGG 1
 #endif
 
-#ifdef WITH_OGG
+#if WITH_OGG
 #define OGGSO "libextractor_ogg:"
 #else
 #define OGGSO ""
 #endif
 
-#ifdef HAVE_ZLIB
+#if HAVE_ZLIB
 #define QTSO "libextractor_qt:"
 #else
 #define QTSO ""
 #endif
 
+#if HAVE_GLIB
+#define OLESO "libextractor_ole2:"
+#else
+#define OLESO ""
+#endif
 
+/* ATTN: order matters (for performance!) since
+   mime-types can be used to avoid parsing once
+   the type has been established! */
 #define DEFSO \
 "libextractor_html:\
+libextractor_man:\
+libextractor_ps:\
+libextractor_pdf:\
 libextractor_mp3:\
+libextractor_id3v2:\
+libextractor_id3v23:\
+libextractor_id3v24:\
+libextractor_mime:\
+libextractor_tar:\
+libextractor_dvi:\
+libextractor_deb:\
 libextractor_png:\
 libextractor_gif:\
 libextractor_wav:\
 libextractor_real:\
-libextractor_pdf:\
 libextractor_jpeg:\
 libextractor_tiff:\
-libextractor_ps:\
 libextractor_zip:\
 libextractor_rpm:\
-libextractor_mime:\
 libextractor_riff:\
 libextractor_mpeg:\
 libextractor_elf:\
+libextractor_oo:\
 libextractor_asf"
 
-#define DEFAULT_LIBRARIES OGGSO QTSO DEFSO
+#define DEFAULT_LIBRARIES OLESO OGGSO QTSO DEFSO
 
 /* ************library initialization ***************** */
 
@@ -159,48 +181,36 @@ void __attribute__ ((destructor)) le_win_fini(void) {
 }
 #endif
 
-#ifndef USE_NS_MODULE
 static char * old_dlsearchpath = NULL;
 
 /* using libtool, needs init! */
 void __attribute__ ((constructor)) le_ltdl_init(void) {
   int err;
+
+  setlocale(LC_ALL, "");
+  bindtextdomain(PACKAGE, LOCALEDIR);
   err = lt_dlinit ();
   if (err > 0)
     {
 #if DEBUG
-      fprintf (stderr,
-	       "initialization of plugin mechanism failed (%s)!\n",
-	       lt_dlerror ());
+      fprintf(stderr,
+	      _("Initialization of plugin mechanism failed: %s!\n"),
+	      lt_dlerror());
 #endif
       return;
     }
   if (lt_dlgetsearchpath() != NULL)
     old_dlsearchpath = strdup(lt_dlgetsearchpath());
   if (lt_dlgetsearchpath () == NULL)
-    lt_dladdsearchdir ("/usr/lib");
-  else if (strstr (lt_dlgetsearchpath (), "/usr/lib") == NULL)
-    lt_dladdsearchdir ("/usr/lib");
-  if (strstr (lt_dlgetsearchpath (), "/usr/local/lib") == NULL)
-    lt_dladdsearchdir ("/usr/local/lib");
-#ifdef LTDL_SYSSEARCHPATH
-  if (strstr (lt_dlgetsearchpath (), LTDL_SYSSEARCHPATH) == NULL)
-    lt_dladdsearchdir (LTDL_SYSSEARCHPATH);
-#endif
-#ifdef ELIBDIR
-  if (strstr (lt_dlgetsearchpath (), ELIBDIR) == NULL)
-    lt_dladdsearchdir (ELIBDIR);
-#endif
+    lt_dladdsearchdir ("/usr/lib/libextractor");
+  else if (strstr (lt_dlgetsearchpath (), "/usr/lib/libextractor") == NULL)
+    lt_dladdsearchdir ("/usr/lib/libextractor");
+  if (strstr (lt_dlgetsearchpath (), "/usr/local/lib/libextractor") == NULL)
+    lt_dladdsearchdir ("/usr/local/lib/libextractor");
 #ifdef PLUGIN_PATH
   if (strstr (lt_dlgetsearchpath (), PLUGIN_PATH) == NULL)
     lt_dladdsearchdir (PLUGIN_PATH);
 #endif
-#ifdef LTDL_SHLLIBPATH_VAR
-  env = getenv (LTDL_SHLIBPATH_VAR);
-  if (env != NULL)
-    if (strstr (lt_dlgetsearchpath (), env) == NULL)
-      lt_dladdsearchdir (env);
-#endif
 }
 
 void __attribute__ ((destructor)) le_ltdl_fini(void) {
@@ -210,8 +220,6 @@ void __attribute__ ((destructor)) le_ltdl_fini(void) {
   lt_dlexit ();
 }
 
-#endif
-
 
 /**
  * Load the default set of libraries. The default set of
@@ -252,7 +260,7 @@ const char *
 EXTRACTOR_getKeywordTypeAsString(const EXTRACTOR_KeywordType type)
 {
   if ((type >= 0) && (type < HIGHEST_TYPE_NUMBER))
-    return keywordTypes[type];
+    return gettext(keywordTypes[type]);
   else
     return NULL;
 }
@@ -314,13 +322,6 @@ static void *getSymbolWithPrefix(void *lib_handle,
     + 1 /* for the optional '_' prefix */;
   char *name=malloc(name_size),*first_error;
   void *symbol=NULL;
-#ifdef USE_NS_MODULE
-  NSSymbol *symbol_obj=NULL;
-  NSLinkEditErrors ler;
-  int lerno;
-  const char *lerfile;
-  const char *dyld_error;
-#endif
 
   snprintf(name,
 	   name_size,
@@ -328,41 +329,15 @@ static void *getSymbolWithPrefix(void *lib_handle,
 	   lib_name,
 	   sym_name);
 
-#ifdef USE_NS_MODULE
-  if (((struct mach_header*)lib_handle)->magic == MH_MAGIC ||
-      ((struct mach_header*)lib_handle)->magic == MH_CIGAM) {
-    if (NSIsSymbolNameDefinedInImage((struct mach_header*)lib_handle,name)) {
-      symbol_obj=NSLookupSymbolInImage((struct mach_header*)lib_handle,name,
-                                       NSLOOKUPSYMBOLINIMAGE_OPTION_BIND |
-                                       NSLOOKUPSYMBOLINIMAGE_OPTION_RETURN_ON_ERROR);
-    }
-  } else {
-    symbol_obj=NSLookupSymbolInModule(lib_handle,name);
-  }
-  if (symbol_obj==NULL) {
-#if DEBUG
-    NSLinkEditError(&ler,&lerno,&lerfile,&dyld_error);
-    if (dyld_error && *dyld_error) {
-      fprintf(stderr,"Could not load %s: NSLookupSymbolInModule() for '%s' failed: %s\n",
-                     lib_name,name,dyld_error);
-    } else {
-      fprintf(stderr,"Could not load %s: NSLookupSymbolInModule() for '%s' failed.\n",
-                     lib_name,name);
-    }
-#endif
-  } else {
-    symbol=NSAddressOfSymbol(symbol_obj);
-  }
-#else
   symbol=lt_dlsym(lib_handle,name+1 /* skip the '_' */);
   if (symbol==NULL) {
     first_error=strdup(lt_dlerror());
     symbol=lt_dlsym(lib_handle,name /* now try with the '_' */);
 #if DEBUG
-    fprintf (stderr,
-             "Resolving symbol '%s' in library %s failed, "
-             "so I tried '%s', but that failed also.  Errors are: "
-             "%s; and then: %s\n",
+    fprintf(stderr,
+	    _("Resolving symbol '%s' in library '%s' failed, "
+	      "so I tried '%s', but that failed also.  Errors are: "
+	      "'%s' and '%s'.\n"),
              name+1,
              lib_name,
              name,
@@ -371,7 +346,6 @@ static void *getSymbolWithPrefix(void *lib_handle,
 #endif
     free(first_error);
   }
-#endif
   free(name);
   return symbol;
 }
@@ -385,130 +359,23 @@ loadLibrary (const char *name,
 	     void **libHandle, 
 	     ExtractMethod * method)
 {
-#ifdef USE_NS_MODULE
-  NSObjectFileImage fileImage;
-  NSObjectFileImageReturnCode returnCode;
-  NSLinkEditErrors ler;
-  int lerno;
-  const char *lerfile;
-  const char *dyld_error;
-  char *file_name;
-
-  file_name
-     = malloc(strlen(PLUGIN_PATH) +
-	      strlen("/") +
-	      strlen(name) +
-	      strlen(".so") + 1);
-  sprintf(file_name,
-	  "%s/%s.so",
-	  PLUGIN_PATH,
-	  name);
-  returnCode
-    = NSCreateObjectFileImageFromFile(file_name,
-				      &fileImage);
-  if (returnCode==NSObjectFileImageInappropriateFile) {
-    NSDestroyObjectFileImage(fileImage);
-    *libHandle=(void*)NSAddImage(file_name,
-				 NSADDIMAGE_OPTION_RETURN_ON_ERROR);
-  } else if (returnCode==NSObjectFileImageSuccess) {
-    *libHandle=NSLinkModule(fileImage,
-                            file_name,
-                            NSLINKMODULE_OPTION_RETURN_ON_ERROR |
-                            NSLINKMODULE_OPTION_PRIVATE |
-                            NSLINKMODULE_OPTION_BINDNOW);
-    
-    NSDestroyObjectFileImage(fileImage);
-  } else {
-#if DEBUG
-    switch (returnCode) {
-    case NSObjectFileImageInappropriateFile:
-      fprintf(stderr,
-	      "Could not load %s: '%s' is not a bundle\n",
-	      name,
-	      file_name);
-      break;
-    case NSObjectFileImageFailure:
-      fprintf(stderr,
-	      "Could not load %s: Object file setup failure in '%s'\n",
-	      name,
-	      file_name);
-      break;
-    case NSObjectFileImageArch:
-      fprintf(stderr,
-	      "Could not load %s: No object for this architecture in '%s'\n",
-	      name,
-	      file_name);
-      break;
-    case NSObjectFileImageFormat:
-      fprintf(stderr,
-	      "Could not load %s: '%s' has bad object file format\n",
-	      name,
-	      file_name);
-      break;
-    case NSObjectFileImageAccess:
-      fprintf(stderr,
-	      "Could not load %s: Cannot read '%s'\n",
-	      name,
-	      file_name);
-      break;
-    default:
-      fprintf(stderr,
-	      "Could not load %s: Could not load '%s' because of some mystery error\n",
-	      name,
-	      file_name);
-      break;
-    }
-#endif
-    NSDestroyObjectFileImage(fileImage);
-    free(file_name);
-    return -1;
-  }
-  
-  if (*libHandle==NULL) {
-    NSLinkEditError(&ler,&lerno,&lerfile,&dyld_error);
-    if (dyld_error && *dyld_error) {
-      fprintf(stderr,
-	      "Could not load %s: NSLinkModule() for '%s' failed: %s\n",
-	      name,file_name,
-	      dyld_error);
-    } else {
-      fprintf(stderr,
-	      "Could not load %s: NSLinkModule() for '%s' failed.\n",
-	      name,
-	      file_name);
-    }
-    free(file_name);
-    return -1;
-  }
-  
-  free(file_name);
-  /* end NS_MODULE */
-#else  
-  /* begin libtool */
-
   *libHandle = lt_dlopenext (name);
   if (*libHandle == NULL)
     {
 #if DEBUG
       fprintf (stderr,
-	       "Loading library %s failed (%s)!\n", 
+	       _("Loading '%s' plugin failed: %s\n"), 
 	       name, 
 	       lt_dlerror ());
 #endif
       return -1;
     }
-#endif
 
   *method = (ExtractMethod) getSymbolWithPrefix (*libHandle, name, "_extract");
-  if (*method == NULL)
-    {
-#ifdef USE_NS_MODULE
-      NSUnLinkModule(*libHandle,0);
-#else
-      lt_dlclose (*libHandle);
-#endif
-      return -1;
-    }
+  if (*method == NULL) {
+    lt_dlclose (*libHandle);
+    return -1;
+  }
   return 1;
 }
 
@@ -596,17 +463,13 @@ EXTRACTOR_removeLibrary(EXTRACTOR_ExtractorList * prev,
 	prev->next = pos->next;
       /* found */
       free (pos->libname);
-#ifdef USE_NS_MODULE
-      NSUnLinkModule(pos->libraryHandle,0);
-#else
       lt_dlclose (pos->libraryHandle);
-#endif
       free (pos);
     }
 #if DEBUG
   else
     fprintf(stderr, 
-	    "Removing library %s failed!\n", 
+	    _("Unloading plugin '%s' failed!\n"), 
 	    library);
 #endif
   return first;
@@ -635,28 +498,39 @@ EXTRACTOR_getKeywords (EXTRACTOR_ExtractorList * extractor,
 		       const char *filename) {
   EXTRACTOR_KeywordList *result;
   int file;
-  char *buffer;
+  char * buffer;
   struct stat fstatbuf;
   size_t size;
 
+#ifdef O_LARGEFILE
+  file = OPEN(filename, O_RDONLY | O_LARGEFILE);
+#else
   file = OPEN(filename, O_RDONLY);
+#endif
   if (-1 == file)
     return NULL;
-  if (-1 == FSTAT(file, &fstatbuf))
-    {
-      close (file);
-      return NULL;
-    }
-  size = fstatbuf.st_size;
-#ifndef MINGW
-  if (size > 0)
-    buffer = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_PRIVATE, file, 0);
-  else {
-    buffer = malloc(1);
-    buffer[0] = '\0';
+  if (-1 == FSTAT(file, &fstatbuf)) {
+    close(file);
+    return NULL;
   }
+  size = fstatbuf.st_size;
+  if (size == 0) {
+    close(file);
+    return NULL;
+  }
+
+#ifndef MINGW
+  if (size > 1* 1024 * 1024 * 1024)
+    size = 1 * 1024 * 1024 * 1024; /* do not mmap/read more than 1 GB! */
+  buffer = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_PRIVATE, file, 0);
 #else
+  if (size > 250 * 1024 * 1024)
+    size = 250 * 1024 * 1024; /* do not malloc/read more than 250 MB! */
   buffer = malloc(size + 1);
+  if (buffer == NULL) { /* out of memory? */
+    close(file);
+    return NULL;
+  }
   buffer[size] = 0;
   if (size > 0)
     read(file, buffer, size);
@@ -806,11 +680,14 @@ EXTRACTOR_printKeywords (FILE * handle,
   while (keywords != NULL)
     {
       if (keywords->keywordType >= HIGHEST_TYPE_NUMBER)
-	fprintf (handle, "INVALID TYPE - %s\n", keywords->keyword);
+	fprintf(handle, 
+		_("INVALID TYPE - %s\n"),
+		keywords->keyword);
       else
-	fprintf (handle,
-		 "%s - %s\n",
-		 keywordTypes[keywords->keywordType], keywords->keyword);
+	fprintf(handle,
+		"%s - %s\n",
+		gettext(keywordTypes[keywords->keywordType]),
+		keywords->keyword);
       keywords = keywords->next;
     }
 }
@@ -877,7 +754,7 @@ EXTRACTOR_extractLastByString (const char * type,
     return result;
   while (keywords != NULL )
     {
-      if ( !strcmp(keywordTypes[keywords->keywordType], type) )
+      if ( !strcmp(gettext(keywordTypes[keywords->keywordType]), type) )
 	result = keywords->keyword;
       keywords = keywords->next;
     }
@@ -907,6 +784,18 @@ EXTRACTOR_countKeywords (EXTRACTOR_KeywordList * keywords)
 
 #ifdef HAVE_JNI_H
 
+#include <jni.h>
+
+/* gcj's jni.h does not define JNIEXPORT/JNICALL (at least
+ * not in my version).  Sun defines it to 'empty' in Linux,
+ * so that should work */
+#ifndef JNIEXPORT
+#define JNIEXPORT
+#endif
+#ifndef JNICALL
+#define JNICALL
+#endif
+
 #include "org_ovmj_libextractor_Extractor.h"
 
 /*
@@ -1014,7 +903,7 @@ JNIEXPORT jstring JNICALL Java_org_ovmj_libextractor_Extractor_getTypeAsStringIn
   const char * str;
   if ( (type < 0) || (type > HIGHEST_TYPE_NUMBER) )
     return NULL; /* error! */  
-  str = keywordTypes[type];
+  str = gettext(keywordTypes[type]);
   if (str == NULL)
     return NULL;
   return (*env)->NewStringUTF(env, 
@@ -1090,6 +979,54 @@ JNIEXPORT jlong JNICALL Java_org_ovmj_libextractor_Extractor_loadPlugin(JNIEnv *
   return ret; 
 }
 
-
 #endif
 /* #ifdef HAVE_JNI_H */
+
+
+/* ********************* python support ****************** */
+
+#ifdef HAVE_PYTHON_H
+
+#include <Python.h>
+
+static PyObject * EXTRACTOR_PY_loadDefaultLibraries(PyObject * self) {
+  return Py_BuildValue("l",
+		       (long)EXTRACTOR_loadDefaultLibraries());
+}
+
+static PyObject * EXTRACTOR_PY_removeAll(PyObject * self,
+					 PyObject * args) {
+  long libs;
+
+  PyArg_ParseTuple(args, 
+		   "l", 
+		   &libs);
+  EXTRACTOR_removeAll((EXTRACTOR_ExtractorList*)libs);
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+static PyObject * EXTRACTOR_PY_getKeywordTypeAsString(PyObject * self,
+						      PyObject * args) {
+  unsigned int type;
+
+  PyArg_ParseTuple(args, 
+		   "i", 
+		   &type); 
+  return Py_BuildValue("s",
+		       EXTRACTOR_getKeywordTypeAsString((EXTRACTOR_KeywordType)type));
+}
+
+static PyMethodDef ExtractorMethods[] = {
+  { "getKeywordTypeAsString", EXTRACTOR_PY_getKeywordTypeAsString,  METH_VARARGS,
+    "Comment." },
+  { NULL, NULL, 0, NULL }
+};
+
+PyMODINIT_FUNC 
+initextractor() {
+  Py_InitModule("extractor", ExtractorMethods);
+}
+
+
+#endif
