@@ -16,19 +16,12 @@
      along with libextractor; see the file COPYING.  If not, write to the
      Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
- **/
+ */
+
+#include "platform.h"
+#include "extractor.h"
 
-#include <extractor.h>
-#include <stdlib.h>
-#include <string.h>
 #define HAVE_MAIN 0
-#if HAVE_MAIN
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <sys/mman.h>
-#endif
-
 
 static void addKeyword(struct EXTRACTOR_Keywords ** list,
 		       const char * keyword,
@@ -545,9 +538,11 @@ struct EXTRACTOR_Keywords * libextractor_elf_extract(char * filename,
 				  dyn.d_un.d_val);
 	  /* "source" of the dependencies: path
 	     to dynamic libraries */
-	  addKeyword(&prev,
-		     rpath,
-		     EXTRACTOR_SOURCE);
+	  if (rpath != NULL) {
+	    addKeyword(&prev,
+		       rpath,
+		       EXTRACTOR_SOURCE);
+	  }
 	  break;
 	}  
 	case DT_NEEDED: {
@@ -558,9 +553,11 @@ struct EXTRACTOR_Keywords * libextractor_elf_extract(char * filename,
 				  &ehdr,
 				  stringIdx,
 				  dyn.d_un.d_val);
-	  addKeyword(&prev,
-		     needed,
-		     EXTRACTOR_DEPENDENCY);
+	  if (needed != NULL) {
+	    addKeyword(&prev,
+		       needed,
+		       EXTRACTOR_DEPENDENCY);
+	  }
 	  break;
 	}  
 	}
@@ -572,8 +569,6 @@ struct EXTRACTOR_Keywords * libextractor_elf_extract(char * filename,
   return prev;
 }
 
-
-
 #if HAVE_MAIN
 int main (int argc, char **argv) {
   int file;
