@@ -21,17 +21,10 @@
      Mooneer Salem (mooneer@translator.cs).  The main changes
      to libhtmlparse were the removal of globals to make the
      code reentrant.
- **/
+ */
 
+#include "platform.h"
 #include "extractor.h"
-#include <string.h>
-#include <stdio.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <errno.h>
 
 #define HAVE_MAIN 0
 
@@ -136,7 +129,7 @@ static struct ArgvTable *addArgToTable(struct ArgvTable *args, char *arg, char *
   }  
   if (args == NULL) {
     fprintf(stderr,
-	    "FATAL: could not allocate: %s at %s:%d\n",
+	    _("Fatal: could not allocate (%s at %s:%d).\n"),
 	    strerror(errno),
 	    __FILE__, __LINE__);
     exit(EXIT_FAILURE);
@@ -477,7 +470,8 @@ static const char *parseXML(const char *html, struct PC_ * pc) {
 
 static const char *parseStartTag (const char *html, struct PC_ * pc) {
   char *tag, *name, *value;
-  const char *tmp;
+  const char * tmp;
+  const char * start = html;
   int ret = 0;
   struct ArgvTable *tmp2 = NULL;
   
@@ -609,7 +603,7 @@ static const char *parseStartTag (const char *html, struct PC_ * pc) {
       } else if (*html == '"') {
 	html++;
 	tmp = html;
-	while(*html != '\0' && 
+	while (*html != '\0' && 
 	      !(*html == '"' && *(html-1) != '\\')) 
 	  html++;
 	value = (char *) calloc(1, (size_t)(html-tmp+1));
@@ -654,16 +648,18 @@ static const char *parseStartTag (const char *html, struct PC_ * pc) {
   }
   if (*html != '\0') html++;
   
-  if (pc->startCallBack != NULL &&  *(html-2)!='/' ) {
-    ret = pc->startCallBack(tag, tmp2, pc->numArgs, pc);
-  }
-  if (pc->endCallBack != NULL && ret==0 && *(html-2)=='/' 
-      && !(pc->XHTMLCallBack)) {
-    ret = pc->endCallBack(tag, pc);
-  }
-  /* these tags may have arguments too, e.g. <hr noshade/> */
-  if (pc->XHTMLCallBack != NULL && *(html-2)=='/') {
-    ret = pc->XHTMLCallBack(tag, tmp2, pc->numArgs, pc);
+  if (html - start > 2) {
+    if (pc->startCallBack != NULL && (*(html-2)!='/')) {
+      ret = pc->startCallBack(tag, tmp2, pc->numArgs, pc);
+    }
+    if (pc->endCallBack != NULL && ret==0 && *(html-2)=='/' 
+	&& !(pc->XHTMLCallBack)) {
+      ret = pc->endCallBack(tag, pc);
+    }
+    /* these tags may have arguments too, e.g. <hr noshade/> */
+    if (pc->XHTMLCallBack != NULL && *(html-2)=='/') {
+      ret = pc->XHTMLCallBack(tag, tmp2, pc->numArgs, pc);
+    }
   }
   if(tmp2 != NULL){
     freeArgs(tmp2, pc);
@@ -930,12 +926,14 @@ static void parse (const char *html, struct PC_ * pc) {
       }
       tmp = (char *) malloc((size_t)(text-html+1));
       if (tmp == NULL) {
-	fprintf(stderr, "WARNING [libhtmlparse]: memory error\n");
+	fprintf(stderr, 
+		_("Fatal: could not allocate (%s at %s:%d).\n"),
+		strerror(errno), __FILE__, __LINE__);
 	return;
       }
       
       strncpy(tmp, html, (size_t)(text-html));
-      
+      tmp[text-html] = '\0';  /* strncpy does not zero-terminate! */
       if (pc->textCallBack != NULL) {
 	int ret = pc->textCallBack(tmp, pc);
 	if (ret != 0) {
@@ -1033,10 +1031,13 @@ static int hasTag(char * arg,
 		  struct ArgvTable * args,
 		  int numargs) {
   int i;
-  for (i=0;i<numargs;i++) 
-    if (0 == strcasecmp(args[i].arg, arg))
-      if (0 == strcasecmp(args[i].val, val))
-	return 1;
+  for (i=0;i<numargs;i++) {
+    if ( (NULL != args[i].arg) &&
+	 (NULL != args[i].val) &&
+	 (0 == strcasecmp(args[i].arg, arg)) &&
+	 (0 == strcasecmp(args[i].val, val)) )
+      return 1;
+  }
   return 0;
 }
 
@@ -1066,12 +1067,12 @@ static struct {
 
 
 
-static int starttag (char *tag, 
-		     struct ArgvTable *args, 
-		     int numargs,
-		     struct PC_ * pc) {
+static int starttag(char *tag, 
+		    struct ArgvTable *args, 
+		    int numargs,
+		    struct PC_ * pc) {
   int i;
-  
+
   if (0 == strcasecmp(tag,"title")) {
     pc->nextTextAction = 1;
     pc->nextKeywordType = EXTRACTOR_TITLE;
@@ -1097,15 +1098,61 @@ static int starttag (char *tag,
     return 0;
   } 
 #endif  
+  if ( (tag != NULL) &&
+       ( (0 == strcasecmp(tag, "body")) ||
+	 (0 == strcasecmp(tag, "/body")) ) )
+    return 1;
   return 0;
 }
 
 static int endtag (char *tag, struct PC_ * pc) {
   pc->nextTextAction = 0;
+  if ( (tag != NULL) &&
+       ( (0 == strcasecmp(tag, "head")) ||
+	 (0 == strcasecmp(tag, "/head")) ) )
+    return 1;
   return 0;
 }
 
 
+/* which mime-types should not be subjected to
+   the HTML extractor (no use trying & parsing
+   is expensive!) */
+static char * blacklist[] = {
+  "image/jpeg",
+  "image/gif",
+  "image/png",
+  "image/x-png",
+  "image/xcf",
+  "image/tiff",
+  "application/java",
+  "application/pdf",
+  "application/postscript",
+  "application/elf",
+  "application/gnunet-directory",
+  "application/x-gzip",
+  "application/bz2",
+  "application/x-rpm",
+  "application/x-rar",
+  "application/x-zip",
+  "application/x-arj",
+  "application/x-compress",
+  "application/x-tar",
+  "application/x-lha",
+  "application/x-gtar",
+  "application/x-dpkg",
+  "application/ogg",
+  "audio/real",
+  "audio/x-wav",
+  "audio/avi",
+  "audio/midi",
+  "audio/mpeg",
+  "video/real",
+  "video/asf",
+  "video/quicktime",
+  NULL,
+};
+
 /* mimetype = text/html */
 struct EXTRACTOR_Keywords * libextractor_html_extract(const char * filename,
                                                       char * data,
@@ -1114,9 +1161,23 @@ struct EXTRACTOR_Keywords * libextractor_html_extract(const char * filename,
   char backup;
   ParserContext pc;
   size_t xsize;
+  const char * mime;
 
   if (size == 0)
     return prev;
+
+  mime = EXTRACTOR_extractLast(EXTRACTOR_MIMETYPE,
+			       prev);
+  if (mime != NULL) {
+    int j;
+    j = 0;
+    while (blacklist[j] != NULL) {
+      if (0 == strcmp(blacklist[j], mime))
+	return prev;
+      j++;
+    }
+  }  
+
   memset(&pc,
 	 0,
 	 sizeof(ParserContext));
