@@ -16,13 +16,11 @@
      along with libextractor; see the file COPYING.  If not, write to the
      Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
- **/
+ */
 
+#include "platform.h"
 #include "extractor.h"
-#include <string.h>
-#include <unistd.h>
-#include <ctype.h>
-#include <stdlib.h>
+
 
 static EXTRACTOR_KeywordList * addKeyword(EXTRACTOR_KeywordType type,
 					  char * keyword,
