@@ -16,7 +16,7 @@
      along with libextractor; see the file COPYING.  If not, write to the
      Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
- **/
+ */
 
 /*
  * This file is based on demux_asf from the xine project (copyright follows).
@@ -52,18 +52,13 @@
  *    build_frame_table
  *  free_qt_info
  *
- * $Id: qtextractor.c,v 1.10 2004/06/21 08:54:26 grothoff Exp $
+ * $Id: qtextractor.c,v 1.12 2004/10/05 20:02:08 grothoff Exp $
  *
  */
 
-#include "extractor_util.h"
+#include "platform.h"
+#include "extractor.h"
 
-#include <stdio.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
 #include <zlib.h>
 
 typedef unsigned int qt_atom;
@@ -1233,6 +1228,8 @@ static qt_error parse_reference_atom (reference_t *ref,
   for (i = ATOM_PREAMBLE_SIZE; i < ref_atom_size - 4; i++) {
 
     current_atom_size = BE_32(&ref_atom[i - 4]);
+    if (i + current_atom_size > ref_atom_size)
+      break;
     current_atom = BE_32(&ref_atom[i]);
 
     if (current_atom == RDRF_ATOM) {
