@@ -16,22 +16,10 @@
      along with libextractor; see the file COPYING.  If not, write to the
      Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
- **/
+ */
 
-#include "extractor_util.h"
-
-#include <stdlib.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-#ifndef MINGW
-#include <sys/socket.h>
-#include <netinet/in.h>
-#else
-#include <winsock2.h>
-#include <pthread.h>
-#endif
-#include <time.h>
-#include <string.h>
+#include "platform.h"
+#include "extractor.h"
 
 /* someone was using this non-portable function.  Mac OS X
  * doesn't have it so rpmextractor wouldn't load.  I rewrite
@@ -1119,7 +1107,7 @@ void * headerFreeData( /*@only@*/ /*@null@*/ const void * data, rpmTagType type)
 	    type == RPM_STRING_ARRAY_TYPE ||
 	    type == RPM_I18NSTRING_TYPE ||
 	    type == RPM_BIN_TYPE)
-		free((void *)data);
+	  free((void *)data);
 	/*@=branchstate@*/
     }
     return NULL;
@@ -1437,8 +1425,8 @@ static int copyEntry(const indexEntry entry,
 	} else {
 	    count = entry->length;
 	    *p = (!minMem
-		? memcpy(malloc(count), entry->data, count)
-		: entry->data);
+		  ? memcpy(malloc(count), entry->data, count)
+		  : entry->data);
 	}
 	break;
     case RPM_STRING_TYPE:
@@ -1903,7 +1891,8 @@ int headerNextIterator(HeaderIterator hi,
 		/*@null@*/ /*@out@*/ hTAG_t tag,
 		/*@null@*/ /*@out@*/ hTYP_t type,
 		/*@null@*/ /*@out@*/ hPTR_t * p,
-		/*@null@*/ /*@out@*/ hCNT_t c)
+		/*@null@*/ /*@out@*/ hCNT_t c,
+		       int do_copy)
 	/*@modifies hi, *tag, *type, *p, *c @*/
 {
     Header h = hi->h;
@@ -1926,7 +1915,7 @@ int headerNextIterator(HeaderIterator hi,
     if (tag)
 	*tag = entry->info.tag;
 
-    rc = copyEntry(entry, type, p, c, 0);
+    rc = copyEntry(entry, type, p, c, do_copy);
     
     /* XXX 1 on success */
     return ((rc == 1) ? 1 : 0);
@@ -2905,7 +2894,7 @@ static void headerMergeLegacySigs(Header h, const Header sig)
     /*@-mods@*/ /* FIX: undocumented modification of sig */
     for (hi = headerInitIterator(sig);
     /*@=mods@*/
-        headerNextIterator(hi, &tag, &type, &ptr, &count);
+        headerNextIterator(hi, &tag, &type, &ptr, &count, 0);
         ptr = hfd(ptr, type))
     {
 	switch (tag) {
@@ -3056,12 +3045,12 @@ struct EXTRACTOR_Keywords * libextractor_rpm_extract(char * filename,
   /* FIXME: add platform! */
   if (isSource)
     sprintf(verb,
-	    "SOURCE RPM %d.%d",
+	    _("Source RPM %d.%d"),
 	    major,
 	    minor);
   else
     sprintf(verb,
-	    "BINARY RPM %d.%d",
+	    _("Binary RPM %d.%d"),
 	    major, 
 	    minor);
   prev = addKeyword(EXTRACTOR_UNKNOWN,
@@ -3072,7 +3061,8 @@ struct EXTRACTOR_Keywords * libextractor_rpm_extract(char * filename,
 				 &tag,
 				 &type,
 				 &p,
-				 &c)) {
+				 &c,
+				 0)) {
     i=0;
     while (tests[i].rtype != 0) {   
       if (tests[i].rtype == tag) {
@@ -3164,6 +3154,12 @@ struct EXTRACTOR_Keywords * libextractor_rpm_extract(char * filename,
       }	      
       i++;
     }
+    if ( ( (type == RPM_BIN_TYPE) ||
+	   (type == RPM_I18NSTRING_TYPE) ||
+	   (type == RPM_STRING_ARRAY_TYPE) ) &&
+ 	 (p != NULL) ) {
+      free((void*)p); 
+    }        
   }
   headerFreeIterator(hi);
   headerFree(hdr);
